/*
  ttp229_keyboard.ino
  Code by by LINGIB
  https://www.instructables.com/member/lingib/instructables/
  Last update 22 August 2020

  ----------
  COPYRIGHT
  ----------
  This code is free software: you can redistribute it and/or
  modify it under the terms of the GNU General Public License as published
  by the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This software is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License. If
  not, see <http://www.gnu.org/licenses/>.
*/

// ----- TFT shield definitions
#include "SPI.h"
#include "Adafruit_GFX.h"
#include "Adafruit_ILI9341.h"
#define TFT_CS    8                     // chip select | slave select
#define TFT_RST   9                     // reset
#define TFT_DC    10                    // data/command | register select | A0 (address select)
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC, TFT_RST);   // create "tft" object

// ----- TTP229 keypad definitions
int Clock = A0, Data = A1;              // use any pins
int Key;                                // output from TTP229 keypad
bool Key_held_down = false;             // used to prevent multiple key responses

// ------------------
// setup()
// ------------------
void setup()
{
  // ----- configure serial
  Serial.begin(115200);

  // ----- configure TFT display
  tft.begin();
  tft.setRotation(1);                   // set graphics page to landscape
  tft.fillScreen(ILI9341_BLACK);

  // ----- configure TTP229 keypad
  pinMode(Clock , OUTPUT);
  pinMode(Data, OUTPUT);
  digitalWrite(Clock, HIGH);
  digitalWrite(Data, HIGH);

  // ----- print heading
  tft.setTextColor(ILI9341_WHITE);
  tft.setTextSize(3);                   // 18*27 pixels per character
  tft.setCursor(142, 10);
  tft.setTextSize(2);                   // 12*18 pixels per character
  tft.print("TTP229");
  tft.setCursor(46, 40);
  tft.print("Capacitive Keyboard");

  // ----- configure keypad text
  tft.setTextSize(3);
  tft.setTextColor(ILI9341_GREEN);
  tft.setCursor(16, 100);
}

// ------------------
//  main loop()
// ------------------
void loop()
{
  // ----- scan keyboard
  Key = scanKeys();                     // -1 returned unless a key is pressed

  // ----- action each keypress
  if (Key > 0)                          // check for key press
  {
    if (Key_held_down == false)         // is this the first time around the loop
    {
      Key_held_down = true;             // prevent multiple responses
      Serial.print(Key);                // display key press
      Serial.print('\t');
      mapKeyboard(Key);
    }
  }
  else
  {
    Key_held_down = false;              // the key has been released
  }
}

// ------------------
//  scanKeys()
// ------------------
/*
   TonTouch TTP229 datasheet
   Page 9 of 19
   TP1=1 TP2=0
   Set 16-keys active low
*/
int scanKeys()
{
  // ----- locals
  int key = -1;                       // return value if key not pushed

  // ----- start scan
  digitalWrite(Data, LOW);            // active low start pulse
  delayMicroseconds(93);              // DV=93uS
  digitalWrite(Data, HIGH);
  delayMicroseconds(10);              // Tw=10uS

  // ----- listen for data
  pinMode(Data , INPUT);              // Arduino now listening

  // ----- shift keypad data out of chip
  for (int i = 1; i < 17; i++)        // scan 16 keypads
  {
    digitalWrite(Clock, LOW);         // F_SCL = 10^6/10 = 100KHz
    delayMicroseconds(5);             // 1/2 F_SCL period
    digitalWrite(Clock, HIGH);        // keypad data appears on rising clock-edge
    if (!digitalRead(Data)) key = i;  // record keypad number when data line goes low
    delayMicroseconds(5);             // 1/2 F_SCL period
  }
  delay(2);                           // Tout=2mS

  // ----- housekeeping
  pinMode(Data, OUTPUT);              // get ready for next key press
  digitalWrite(Data, HIGH);

  // ----- return data
  return key;                         // returns -1 if no key pressed
}

// ------------------
//  mapKeyboard()
// ------------------
void mapKeyboard(int key)
{
  switch (key) {
    case 1:
      Serial.println("1");
      tft.print("1");
      break;
    case 2:
      Serial.println("2");
      tft.print("2");
      break;
    case 3:
      Serial.println("3");
      tft.print("3");
      break;
    case 4:
      Serial.println("C");
      tft.print("C");
      break;
    case 5:
      Serial.println("4");
      tft.print("4");
      break;
    case 6:
      Serial.println("5");
      tft.print("5");
      break;
    case 7:
      Serial.println("6");
      tft.print("6");
      break;
    case 8:
      Serial.println("D");
      tft.print("D");
      break;
    case 9:
      Serial.println("7");
      tft.print("7");
      break;
    case 10:
      Serial.println("8");
      tft.print("8");
      break;
    case 11:
      Serial.println("9");
      tft.print("9");
      break;
    case 12:
      Serial.println("E");
      tft.print("E");
      break;
    case 13:
      Serial.println("A");
      tft.print("A");
      break;
    case 14:
      Serial.println("0");
      tft.print("0");
      break;
    case 15:
      Serial.println("B");
      tft.print("B");
      break;
    case 16:
      Serial.println("F");
      tft.print("F");
      break;
    default:
      break;
  }
}
